var Scroller = {

    track: null,
    thumb: null,
    viewport: null,
    inside: null,

    init: function(track, thumb, viewport, inside) {
        this.track = track;
        this.thumb = thumb;
        this.viewport = viewport;
        this.inside = inside || false;

        track.top = track.getElementsByTagName("div")[0];
        track.mid = track.getElementsByTagName("div")[1];
        track.bot = track.getElementsByTagName("div")[2];
        track.width = width(track);
        track.over = false;
        track.mouseY = null;
        track.bearing = null;
        track.timeout = null;
        track.interval = null;
        track.mousedown = function(event) {
            this.mouseY = event.offsetY + event.target.offsetTop;
            this.over = true;

            var top = (Scroller.thumb.style.top) ? parseInt(Scroller.thumb.style.top) : 0;
            this.bearing = (this.mouseY < top) ? -1 : 1;

            this.scroll();
            this.timeout = setTimeout('Scroller.track.interval = setInterval("Scroller.track.scroll()", 150)', 500);

            this.addEventListener("mouseover", this.mouseover, false);
            this.addEventListener("mouseout", this.mouseout, false);

            document.addEventListener("mousemove", this.mousemove, false);
            document.addEventListener("mouseup", this.mouseup, false);
        }
        track.mouseup = function() {
            clearTimeout(Scroller.track.timeout);
            clearInterval(Scroller.track.interval);

            this.removeEventListener("mouseover", this.mouseover, false);
            this.removeEventListener("mouseout", this.mouseout, false);

            document.removeEventListener("mousemove", this.mousemove, false);
            document.removeEventListener("mouseup", this.mouseup, false);
        }
        track.mouseover = function() { Scroller.track.over = true; }
        track.mouseout = function() { Scroller.track.over = false; }
        track.mousemove = function(event) { Scroller.track.mouseY = event.offsetY + event.target.offsetTop; }
        track.scroll = function() {
            var v = Scroller.viewport;
            var c = Scroller.viewport.content;
            var th = Scroller.thumb;

            if (!this.over) return;
            var top = (th.style.top) ? parseInt(th.style.top) : 0;
            var bearing = (this.mouseY < top) ? -1 : 1;
            if (this.bearing != bearing) return;
            if (bearing == 1) {
                if (this.mouseY <= top + height(th)) { this.mouseup(); return; }
            } else {
                if (this.mouseY >= top) { this.mouseup(); return; }
            }
            var y = top + (bearing * Math.floor(height(v) / (height(c) / height(v))));
            Scroller.scroll(0, y);
        }

        thumb.top = thumb.getElementsByTagName("div")[0];
        thumb.mid = thumb.getElementsByTagName("div")[1];
        thumb.bot = thumb.getElementsByTagName("div")[2];

        viewport.content = viewport.getElementsByTagName("div")[0];
        viewport.width = width(viewport);

        track.addEventListener("mousewheel", Scroller.mousewheel, false);
        track.addEventListener("mousedown", Scroller.track.mousedown, false);
        thumb.addEventListener("mousewheel", Scroller.mousewheel, false);
        viewport.content.addEventListener("mousewheel", Scroller.mousewheel, false);
        window.addEventListener("resize", function() { Scroller.update() }, false);

        this.update();
    },

    update: function() {
        var v = this.viewport;
        var c = this.viewport.content;
        var tr = this.track;;
        var th = this.thumb;

        if (this.inside) {
            tr.parentNode.style.top = v.offsetTop + "px";
            tr.parentNode.style.left = v.offsetLeft + v.width - tr.width + "px";
            v.style.width = c.style.width = v.width - tr.width + "px";
        }

        var pages = height(c) / height(v);

        tr.style.height = height(v)+ "px";
        tr.mid.style.top = height(tr.top) + "px";
        tr.mid.style.height = height(v) - height(tr.top) - height(tr.bot) + "px";

        th.mid.style.top = height(th.top) + "px";
        th.mid.style.height = Math.max((height(v) / Math.max(pages,1)) - height(th.top) - height(th.bot), 0) + "px";
        th.style.height = height(th.mid) + height(th.top) + height(th.bot) + "px";

        Drag.init(th, null, 0, 0, 0, (height(v) - height(th.top) - height(th.mid) - height(th.bot)));
        th.onDrag = this.scroll;

        if (height(c) == 0 || height(th) >= height(tr)) {
            tr.parentNode.style.visibility = "hidden";
            if (this.inside) v.style.width = c.style.width = v.width + "px";
            return;
        }

        tr.parentNode.style.visibility = "visible";
        this.scroll(0, -parseInt(c.style.top || 0) / ((height(c) - height(v)) / (height(v) - height(th))));
    },

    mousewheel: function(event) {
        var v = Scroller.viewport;
        var c = Scroller.viewport.content;
        var top = Scroller.thumb.style.top ? parseInt(Scroller.thumb.style.top) : 0;
        var delta = event.wheelDelta / 120 * -1;
        var ratio = height(v) / height(c) * 10;
        var y = Math.round(delta * ratio);
        if (delta < 0 && y == 0) y = -1;
        if (delta > 0 && y == 0) y = +1;
        Scroller.scroll(0, top + y);
    },

    scroll: function(x,y) {
        var v = Scroller.viewport;
        var c = Scroller.viewport.content;
        var th = Scroller.thumb;

        if (y < th.minY) y = th.minY;
        if (y > th.maxY) y = th.maxY;

        c.style.top = (-1 * ((height(c) - height(v)) / (height(v) - height(th))) * y) + "px";

        if (this != th) th.style.top = y + "px";
    }
};

function height(obj) {
    var h = null;
    if (obj.style.height)
        h = obj.style.height;
    else
        h = document.defaultView.getComputedStyle(obj,"").getPropertyValue("height");
    return (h == null) ? 0 : parseInt(h);
}

function width(obj) {
    var w = null;
    if (obj.style.width)
        w = obj.style.width;
    else
        w = document.defaultView.getComputedStyle(obj,"").getPropertyValue("width");
    return (w == null) ? 0 : parseInt(w);
}
